{module, inject} = angular.mock

describe 'simple-search', ->
  $compile = null
  $element = null
  $scope = null
  fakeHttp = null
  fakeWindow = null
  isolate = null

  before ->
    angular.module('h', [])
    .directive('simpleSearch', require('../simple-search'))

  beforeEach module('h')

  beforeEach module ($provide) ->
    fakeHttp = {pendingRequests: []}
    $provide.service('$http', -> fakeHttp)
    return

  beforeEach inject (_$compile_, _$rootScope_) ->
    $compile = _$compile_
    $scope = _$rootScope_.$new()

    $scope.update = sinon.spy()
    $scope.clear = sinon.spy()

    template= '''
    <simple-search
          query="query"
          on-search="update(query)"
          on-clear="clear()">
    </simple-search>
    '''

    $element = $compile(angular.element(template))($scope)

    # add element to document so that it becomes focusable
    # and we get default form behaviors
    document.body.appendChild($element[0])

    $scope.$digest()
    isolate = $element.isolateScope()

  afterEach ->
    document.body.removeChild($element[0])

  it 'updates the search-bar', ->
    $scope.query = "Test query"
    $scope.$digest()
    assert.equal(isolate.searchtext, $scope.query)

  it 'calls the given search function', ->
    isolate.searchtext = "Test query"
    isolate.$digest()
    $element.find('form').triggerHandler('submit')
    assert.calledWith($scope.update, "Test query")

  it 'invokes callbacks when the input model changes', ->
    $scope.query = "Test query"
    $scope.$digest()
    assert.calledOnce($scope.update)
    $scope.query = ""
    $scope.$digest()
    assert.calledOnce($scope.clear)

  it 'adds a class to the form when there is no input value', ->
    $form = $element.find('.simple-search-form')
    assert.include($form.prop('className'), 'simple-search-inactive')

  it 'removes the class from the form when there is an input value', ->
    $scope.query = "Test query"
    $scope.$digest()

    $form = $element.find('.simple-search-form')
    assert.notInclude($form.prop('className'), 'simple-search-inactive')

  it 'sets the `loading` scope key when http requests are in progress', ->
    fakeHttp.pendingRequests = []
    isolate.$digest()
    assert.isFalse(isolate.loading)
    fakeHttp.pendingRequests = ['bogus']
    isolate.$digest()
    assert.isTrue(isolate.loading)

  it 'expands the search field when the input is non-empty', ->
    input = $element.find('.simple-search-input')
    assert.isFalse(input.hasClass('is-expanded'))
    input.val('query')
    input.trigger('change')
    isolate.$digest()
    assert.isTrue(input.hasClass('is-expanded'))

  it 'focuses the search field when clicking the search button', ->
    input = $element.find('.simple-search-input')
    searchBtn = $element.find('button')
    assert.ok(document.activeElement != input[0])
    searchBtn.click()
    assert.ok(document.activeElement == input[0])

  it 'does not update the search when clicking the search button', ->
    searchBtn = $element.find('button')
    input = $element.find('.simple-search-input')
    input.val('query')
    input.trigger('change')
    searchBtn.click()
    assert.notCalled($scope.update)
